package com.slibra.framework.web.service;

import javax.annotation.Resource;

import com.slibra.common.config.ApplicationProperties;
import com.slibra.common.core.domain.model.LoginBody;
import com.slibra.common.enums.BusinessEnum;
import com.slibra.common.enums.UserStatus;
import com.slibra.common.utils.third.AliSMSUtil;
import com.slibra.common.utils.third.NumUtil;
import com.slibra.common.utils.uuid.IdUtils;
import com.slibra.framework.security.authentication.SmsCodeAuthenticationToken;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;
import com.slibra.common.constant.CacheConstants;
import com.slibra.common.constant.Constants;
import com.slibra.common.constant.UserConstants;
import com.slibra.common.core.domain.entity.SysUser;
import com.slibra.common.core.domain.model.LoginUser;
import com.slibra.common.core.redis.RedisCache;
import com.slibra.common.exception.ServiceException;
import com.slibra.common.exception.user.BlackListException;
import com.slibra.common.exception.user.CaptchaException;
import com.slibra.common.exception.user.CaptchaExpireException;
import com.slibra.common.exception.user.UserNotExistsException;
import com.slibra.common.exception.user.UserPasswordNotMatchException;
import com.slibra.common.utils.DateUtils;
import com.slibra.common.utils.MessageUtils;
import com.slibra.common.utils.StringUtils;
import com.slibra.common.utils.ip.IpUtils;
import com.slibra.framework.manager.AsyncManager;
import com.slibra.framework.manager.factory.AsyncFactory;
import com.slibra.framework.security.context.AuthenticationContextHolder;
import com.slibra.system.service.ISysConfigService;
import com.slibra.system.service.ISysUserService;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import static com.slibra.common.constant.MyConstants.LOGIN_TYPE_APPEND;

/**
 * 登录校验方法
 * 
 *
 */
@Component
@Slf4j
public class SysLoginService
{
    @Autowired
    private TokenService tokenService;

    @Resource
    private AuthenticationManager authenticationManager;

    @Autowired
    private RedisCache redisCache;
    
    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private ApplicationProperties properties ;

    // 是否允许账户多终端同时登录（true允许 false不允许）
    @Value("${token.soloLogin}")
    private boolean soloLogin;

    /**
     * 登录验证
     * 
     * @param username 用户名
     * @param password 密码
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public String login(String username, String password, String code, String uuid, Integer type)
    {
        // 验证码校验
//        validateCaptcha(username, code, uuid);
        //做了多环境区分 目前只能在sql层处理了
        return authUserByNamePwdAndGenToken(username + LOGIN_TYPE_APPEND + type, password);
    }

    public String authUserByNamePwdAndGenToken(String username, String password) {
        // 登录前置校验
        loginPreCheck(username, password);
        //认证
        Authentication authentication = null;
        try
        {
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(username, password);
            AuthenticationContextHolder.setContext(authenticationToken);
            // 该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        }
        catch (Exception e)
        {
            if (e instanceof BadCredentialsException)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
                throw new UserPasswordNotMatchException();
            }
            else
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, e.getMessage()));
                throw new ServiceException(e.getMessage());
            }
        }
        finally
        {
            AuthenticationContextHolder.clearContext();
        }
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        recordLoginInfo(loginUser.getUserId());
        //2024年12月18日16:56:45 账户不允许多终端登录
        if (!soloLogin)
        {
            // 如果用户不允许多终端同时登录，清除缓存信息
            String userIdKey = Constants.LOGIN_USERID_KEY + loginUser.getUser().getUserId();
            String userKey = redisCache.getCacheObject(userIdKey);
            if (StringUtils.isNotEmpty(userKey))
            {
                redisCache.deleteObject(userIdKey);
                redisCache.deleteObject(userKey);
            }
        }
        // 生成token
        return tokenService.createToken(loginUser);
    }

    /**
     * 校验验证码
     * 
     * @param username 用户名
     * @param code 验证码
     * @param uuid 唯一标识
     * @return 结果
     */
    public void validateCaptcha(String username, String code, String uuid)
    {
        boolean captchaEnabled = configService.selectCaptchaEnabled();
        if (captchaEnabled)
        {
            String verifyKey = CacheConstants.CAPTCHA_CODE_KEY + StringUtils.nvl(uuid, "");
            String captcha = redisCache.getCacheObject(verifyKey);
            redisCache.deleteObject(verifyKey);
            if (captcha == null)
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.expire")));
                throw new CaptchaExpireException();
            }
            if (!code.equalsIgnoreCase(captcha))
            {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.error")));
                throw new CaptchaException();
            }
        }
    }

    /**
     * 登录前置校验
     * @param username 用户名
     * @param password 用户密码
     */
    public void loginPreCheck(String username, String password)
    {
        // 用户名或密码为空 错误
        if (StringUtils.isEmpty(username) || StringUtils.isEmpty(password))
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("not.null")));
            throw new UserNotExistsException();
        }
        // 密码如果不在指定范围内 错误
        if (password.length() < UserConstants.PASSWORD_MIN_LENGTH
                || password.length() > UserConstants.PASSWORD_MAX_LENGTH)
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
            throw new UserPasswordNotMatchException();
        }
        // 用户名不在指定范围内 错误
        if (username.length() < UserConstants.USERNAME_MIN_LENGTH
                || username.length() > UserConstants.USERNAME_MAX_LENGTH)
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
            throw new UserPasswordNotMatchException();
        }
        // IP黑名单校验
        String blackStr = configService.selectConfigByKey("sys.login.blackIPList");
        if (IpUtils.isMatchedIp(blackStr, IpUtils.getIpAddr()))
        {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(username, Constants.LOGIN_FAIL, MessageUtils.message("login.blocked")));
            throw new BlackListException();
        }
    }

    /**
     * 记录登录信息
     *
     * @param userId 用户ID
     */
    public void recordLoginInfo(Long userId)
    {
        SysUser sysUser = new SysUser();
        sysUser.setUserId(userId);
        sysUser.setLoginIp(IpUtils.getIpAddr());
        sysUser.setLoginDate(DateUtils.getNowDate());
        //2024年08月12日09:05:40 因为用户前后端类型默是前端用户，因为没有传过来，默认就是前端了，这里该字段不做更新
        sysUser.setType(null);
        userService.updateUserProfile(sysUser);
    }


    public String sendSmsCode(String phoneNumber) {
        // 手机号码
        phoneNumber = phoneNumber.trim();
        // 校验手机号
//        SysUser user = userService.selectUserByPhonenumber(phoneNumber);
        //多环境区分，目前这些只涉及到前端接口，所以目前在此写死
        SysUser user = userService.selectUserByPhonenumber(phoneNumber  + LOGIN_TYPE_APPEND + BusinessEnum.LoginUserTypeEnum.front.getCode());
        if (StringUtils.isNull(user)) {
            throw new ServiceException("登录用户：" + phoneNumber+ " 不存在");
        }else if (UserStatus.DELETED.getCode().equals(user.getDelFlag())) {
            throw new ServiceException("对不起，您的账号：" + phoneNumber+ " 已被删除");
        }else if (UserStatus.DISABLE.getCode().equals(user.getStatus())) {
            throw new ServiceException("对不起，您的账号：" + phoneNumber+ " 已停用");
        }

        /**
         * 省略一千万行代码：校验发送次数，一分钟内只能发1条，一小时最多5条，一天最多10条，超出提示前端发送频率过快。
         * 登录第三方短信平台后台，康康是否可以设置短信发送频率，如果有也符合业务需求可以不做处理。
         */

        // 发送短信（实际按系统业务实现）
        // 生成短信验证码
        String code = NumUtil.get6Long();
        log.info("手机号：{}\n发送的验证码为：{}", phoneNumber, code);
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put("code", code);
        //短信下发(sendMsg()方法做了判断，所以这里无需判断)
        AliSMSUtil.sendMsg(properties.getAliCloud().getTemplateCodeMessage(), phoneNumber, paramMap);
        /*String result = AliSMSUtil.sendMsg(properties.getAliCloud().getTemplateCodeMessage(), phoneNumber, paramMap);
        if(HttpStatus.SUCCESS != JSON.parseObject(result).getInteger("httpStatus"))
            throw new ServiceException("短信下发失败");*/

        // 保存redis缓存
        String uuid = IdUtils.simpleUUID();
        String verifyKey = CacheConstants.REDIS_KEY_SMSLOGIN_SMSCODE + uuid;
        redisCache.setCacheObject(verifyKey, code, CacheConstants.REDIS_EXPIRATION_SMSLOGIN_SMSCODE, TimeUnit.MINUTES);
        return uuid;
    }

    public String smsLogin(LoginBody loginBody) {
        // 手机号码
        String phoneNumber = loginBody.getPhoneNumber();
        // 校验验证码
        String verifyKey = CacheConstants.REDIS_KEY_SMSLOGIN_SMSCODE + loginBody.getUuid();
        String captcha = redisCache.getCacheObject(verifyKey);
        if(captcha == null) {
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(phoneNumber, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.expire")));
            // 抛出一个验证码过期异常
            throw new CaptchaExpireException();
        }
        if(!captcha.equals(loginBody.getSmsCode().trim())){
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(phoneNumber, Constants.LOGIN_FAIL, MessageUtils.message("user.jcaptcha.error")));
            // 抛出一个验证码错误的异常
            throw new CaptchaException();
        }
        redisCache.deleteObject(verifyKey);

        // 用户验证
        Authentication authentication = null;
        try {
            // 该方法会去调用UserDetailsByPhonenumberServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(new SmsCodeAuthenticationToken(phoneNumber));
        } catch (Exception e) {
            if (e instanceof BadCredentialsException) {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(phoneNumber,
                        Constants.LOGIN_FAIL, MessageUtils.message("account.not.incorrect")));
                throw new UserPasswordNotMatchException();
            } else {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(phoneNumber,
                        Constants.LOGIN_FAIL, e.getMessage()));
                throw new ServiceException(e.getMessage());
            }
        }
        // 执行异步任务，记录登录信息
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(phoneNumber,
                Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        // 获取登录人信息
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        // 修改最近登录IP和登录时间
        recordLoginInfo(loginUser.getUserId());
        // 生成token
        String token = tokenService.createToken(loginUser);
        return token;
    }


}
