package com.slibra.business.service.impl;

import java.util.*;
import java.util.stream.Collectors;

import com.slibra.business.domain.City;
import com.slibra.business.domain.TNeighborhoodBuilding;
import com.slibra.business.domain.TPumpingStation;
import com.slibra.business.mapper.CityMapper;
import com.slibra.business.mapper.TNeighborhoodBuildingMapper;
import com.slibra.business.res.PumpAndBuildingNumberNames;
import com.slibra.business.res.PumpAndNumber;
import com.slibra.common.utils.DateUtils;
import com.slibra.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.slibra.business.mapper.TNeighborhoodMapper;
import com.slibra.business.domain.TNeighborhood;
import com.slibra.business.service.ITNeighborhoodService;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

/**
 * 服务小区Service业务层处理
 * 
 * @author slibra
 * @date 2024-11-05
 */
@Service
public class TNeighborhoodServiceImpl implements ITNeighborhoodService 
{
    @Autowired
    private TNeighborhoodMapper tNeighborhoodMapper;

    @Autowired
    private TNeighborhoodBuildingMapper tNeighborhoodBuildingMapper;

    @Autowired
    private CityMapper cityMapper;

    /**
     * 查询服务小区
     * 
     * @param id 服务小区主键
     * @return 服务小区
     */
    @Override
    public TNeighborhood selectTNeighborhoodById(Long id)
    {
        return tNeighborhoodMapper.selectTNeighborhoodById(id);
    }

    /**
     * 查询服务小区列表
     * 
     * @param tNeighborhood 服务小区
     * @return 服务小区
     */
    @Override
    public List<TNeighborhood> selectTNeighborhoodList(TNeighborhood tNeighborhood)
    {
        List<TNeighborhood> tNeighborhoods = tNeighborhoodMapper.selectTNeighborhoodList(tNeighborhood);
        if(!CollectionUtils.isEmpty(tNeighborhoods)){
            for (TNeighborhood neighborhood : tNeighborhoods) {
                //2024年11月12日10:58:29  额外返回楼号 关联泵站的信息
                Long id = neighborhood.getId();//小区ID
                neighborhood.setBuildingsNames(this.tNeighborhoodMapper.getNeighbourhoodBuildingsNamesById(id));
                neighborhood.setPumpingStationNames(this.tNeighborhoodMapper.getPumpingStationNamesById(id));
                this.addExtraAddress(neighborhood);
            }
        }
        return tNeighborhoods;
    }

    /**
     * 额外处理省市区的名字
     * @param neighborhood
     */
    private void addExtraAddress(TNeighborhood neighborhood) {
        String provinceId = neighborhood.getProvinceId();
        String cityId = neighborhood.getCityId();
        String countryId = neighborhood.getCountryId();
        //省
        if(StringUtils.isNotBlank(provinceId)){
            City city = this.cityMapper.selectCityById(provinceId);
            if(!Objects.isNull(city)){
                neighborhood.setProvinceName(city.getName());
            }
        }
        //市
        if(StringUtils.isNotBlank(cityId)){
            City city = this.cityMapper.selectCityById(cityId);
            if(!Objects.isNull(city)){
                neighborhood.setCityName(city.getName());
            }
        }
        //区县
        if(StringUtils.isNotBlank(countryId)){
            City city = this.cityMapper.selectCityById(countryId);
            if(!Objects.isNull(city)){
                neighborhood.setCountryName(city.getName());
            }
        }
    }

    /**
     * 新增服务小区
     * 
     * @param tNeighborhood 服务小区
     * @return 结果
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int insertTNeighborhood(TNeighborhood tNeighborhood)
    {
        tNeighborhood.setCreateTime(DateUtils.getNowDate());
        int i = tNeighborhoodMapper.insertTNeighborhood(tNeighborhood);
        //2024年11月12日15:08:28  需要额外处理一下关联表（小区和楼号的关联表）
        this.handleBuildingNumber(tNeighborhood);
        return i;
    }

    private void handleBuildingNumber(TNeighborhood tNeighborhood) {
        //先把历史的楼号删除
        Long id = tNeighborhood.getId();
        deleteNeighbourhoodBuildingsById(id);
        //再把新的楼号添加进去
        List<String> buildingNameList = tNeighborhood.getBuildingNameList();
        for (int i = 0; i < buildingNameList.size(); i++) {
            TNeighborhoodBuilding tNeighborhoodBuilding = new TNeighborhoodBuilding();
            tNeighborhoodBuilding.setNeighborhoodId(id);
            tNeighborhoodBuilding.setName(buildingNameList.get(i));
            tNeighborhoodBuilding.setSort(i);
            this.tNeighborhoodBuildingMapper.insertTNeighborhoodBuilding(tNeighborhoodBuilding);
        }
    }

    /**
     * 通过小区的ID把之前设置的楼号的信息删除
     * @param id
     */
    private void deleteNeighbourhoodBuildingsById(Long id) {
        List<TNeighborhoodBuilding> tNeighborhoodBuildings = this.tNeighborhoodBuildingMapper.selectTNeighborhoodBuildingList(TNeighborhoodBuilding.builder().neighborhoodId(id).build());
        if(!CollectionUtils.isEmpty(tNeighborhoodBuildings)){
            this.tNeighborhoodBuildingMapper.deleteTNeighborhoodBuildingByIds(tNeighborhoodBuildings.stream().map(TNeighborhoodBuilding::getId).toArray(Long[]::new));
        }
    }

    /**
     * 修改服务小区
     * 
     * @param tNeighborhood 服务小区
     * @return 结果
     */
    @Override
    public int updateTNeighborhood(TNeighborhood tNeighborhood)
    {
        tNeighborhood.setUpdateTime(DateUtils.getNowDate());
        //2024年11月12日15:08:28  需要额外处理一下关联表（小区和楼号的关联表）
        this.handleBuildingNumber(tNeighborhood);
        return tNeighborhoodMapper.updateTNeighborhood(tNeighborhood);
    }

    /**
     * 批量删除服务小区
     * 
     * @param ids 需要删除的服务小区主键
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteTNeighborhoodByIds(Long[] ids)
    {
        for (Long id : ids) {
            deleteNeighbourhoodBuildingsById(id);
        }
        return tNeighborhoodMapper.deleteTNeighborhoodByIds(ids);
    }

    /**
     * 删除服务小区信息
     * 
     * @param id 服务小区主键
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteTNeighborhoodById(Long id)
    {
        //2024年11月12日15:27:10 删除小区的同时，需要把对应小区楼号也同时删除
        deleteNeighbourhoodBuildingsById(id);
        return tNeighborhoodMapper.deleteTNeighborhoodById(id);
    }

    @Override
    public List<PumpAndNumber> getPumpingStationAndNeighbourhoodBuildingsById(Long id) {
        //先通过小区的ID查询所有的关联的泵站的信息
        /*List<TPumpingStation> pumpingStations = this.tNeighborhoodMapper.getPumpingStationByNeighbourhoodId(id);
        if(CollectionUtils.isEmpty(pumpingStations))
            return Collections.emptyList();
        List<PumpAndNumber> result = new ArrayList<>(pumpingStations.size());
        for (TPumpingStation pumpingStation : pumpingStations) {
            Long pumpingStationId = pumpingStation.getId();
            PumpAndNumber pumpAndNumber = new PumpAndNumber();
            pumpAndNumber.setPumpingStationId(pumpingStationId);
            pumpAndNumber.setPumpingStationName(pumpingStation.getName());
            pumpAndNumber.setBuildingsNames(this.tNeighborhoodMapper.getNeighbourhoodBuildingNamesByPumpingStationId(pumpingStationId));
            result.add(pumpAndNumber);
        }
        return result;*/
        //2024年11月12日14:03:17 上述的查询太浪费性能，改成一次查出多条对应的数据，再在程序中处理
        List<PumpAndBuildingNumberNames> pumpAndNumbers = this.tNeighborhoodMapper.getPumpAndBuildingNumberNamesByNeighbourhoodId(id);
        if(CollectionUtils.isEmpty(pumpAndNumbers))
            return Collections.emptyList();
        Map<String, List<PumpAndBuildingNumberNames>> collect = pumpAndNumbers.stream().collect(Collectors.groupingBy(PumpAndBuildingNumberNames::getPumpingStationName));
        List<PumpAndNumber> result = new ArrayList<>(collect.size());
        collect.forEach((key, value) -> {
            PumpAndNumber pumpAndNumber = new PumpAndNumber();
            pumpAndNumber.setPumpingStationName(key);
            pumpAndNumber.setBuildingsNames(value.stream().map(PumpAndBuildingNumberNames::getNeighborhoodBuildingName).collect(Collectors.joining("、")));
            result.add(pumpAndNumber);
        });
        return result;
    }

    public static void main(String[] args) {
        List<TNeighborhoodBuilding> tNeighborhoodBuildings = new ArrayList<>();
        tNeighborhoodBuildings.add(TNeighborhoodBuilding.builder().id(1L).build());
        tNeighborhoodBuildings.add(TNeighborhoodBuilding.builder().id(2L).build());
        tNeighborhoodBuildings.add(TNeighborhoodBuilding.builder().id(3L).build());
        Long[] array = tNeighborhoodBuildings.stream().map(TNeighborhoodBuilding::getId).toArray(Long[]::new);
        System.out.println(Arrays.toString(array));
    }
}
